function [distances, distances_incomplete,distances_sampled, success] = compute_distance_matrix(...
  image_function, number_shapes, save_filename_prefix, options, convergence_error_function, desired_shape_space_dimensionality, useCurrentResults, matCompletionFunctionString)
  % [result, success] = compute_distance_matrix(image_function, number_shapes, save_filename_prefix, registration_options, convergence_error_function, desired_shape_space_dimensionality, maximum_parallel_cpus)
  %
  %   Computes the LDDMM distance between each pair of shapes and returns a distance matrix in result.
  %   Parallel.
  %   
  %   2012-12-01 tebuck: copied from register_shapes_to_mean_shape.m.
  %   2013-04-20 tebuck: added argument  to stop when the number of distances known is the same as the number of degrees of freedom of a distance matrix for points of a specified dimensionality.
  %   2013-04-24 tebuck: adding parfor if new argument maximum_parallel_cpus is greater than one.
  %   2013-05-13 tebuck: skipped distances are now silent as those messages were mostly noise.
  %   2013-08-14 gj:     Removed skipping distance matrix if file already
  %                      exists, so we can add images and update the shape 
  %                      as preprocessed results complete
  %   2013-08-18 gj:     Distance matrix fills randomly to less bias reconstruct_distance_matrix
  %   2013-08-28 gj:     Added return statement in catch block of
  %                      compute_single_distance, and removed parfor loop
  %   2013-08-29 gj:     Added "useCurrentResults" to generate a model with
  %                      only current results, and fixed bug where .tmp
  %                      files were being left behind
  % Aug 30, 2013 G. Johnson    Changed they way files are input into the
%                            diffeomorphic model function
  % Sept 4, 2013 G. Johnson Improved error handling

  
    if ~exist('desired_shape_space_dimensionality', 'var')
    desired_shape_space_dimensionality = inf;
    end

    if ~exist('useCurrentResults', 'var')
    useCurrentResults = false;
    end
    
    if ~exist('matCompletionFunctionString', 'var') | isempty(matCompletionFunctionString)
        matCompletionFunctionString = 'defaultCompletion(distances_sampled, dist_nans)';
    end

    distances_sparse_file = [save_filename_prefix filesep 'distances_sparse.mat'];
    
    distances_save_dir = [save_filename_prefix filesep 'distances'];
    
    if ~exist(distances_save_dir, 'dir')
        mkdir(distances_save_dir)
    end
    
   
    [distances_incomplete, distances_sampled] = loadCalculatedDistances(number_shapes, distances_save_dir, distances_sparse_file);
    
    success = true;
    isdone = false;
    
    if useCurrentResults
        isdone = true;
    end
    
    dist_nans = [];
    nancounter = 1;
    % Either compute distances in series or just load them and check success using this same loop:
    while ~isdone
        
        [ind1, ind2, isdone] = eval(matCompletionFunctionString);
        
        if isdone
            continue;
        end
        
        disp(['Computing distance ( ' num2str(ind1) ' , ' num2str(ind2) ')']);
        
        [distance, individual_success] = compute_single_distance(image_function, distances_save_dir, options, convergence_error_function, desired_shape_space_dimensionality, ind1, ind2, useCurrentResults);
        
        if isnan(distance)
            dist_nans(nancounter,:) = [ind1, ind2];
            nancounter = nancounter+1;
        else
            [distances_incomplete, distances_sampled] = addCalculatedDistance(ind1, ind2, distance, distances_sparse_file);
        end
        % Distance matrix is not complete if one distance is missing:
        success = success && individual_success;
    end
%     save([save_filename_prefix filesep 'distances_incomplete.mat'], 'distances_incomplete', 'distances_sampled');
%     
%     
    distances_incomplete = full(distances_incomplete) + full(distances_incomplete)';
    distances_incomplete((full(distances_sampled) + full(distances_sampled)')==0) = NaN;

    distances = reconstruct_distance_matrix2(distances_incomplete, desired_shape_space_dimensionality);
  
    if success
        save([save_filename_prefix filesep 'distances.mat'], 'distances', 'success', 'distances_sampled');
    end


end

function [distance, success] = compute_single_distance(...
  image_function, save_filename_prefix, registration_options, convergence_error_function, desired_shape_space_dimensionality, shape_index, shape_index2, useCurrentResults)

  % Only compute the first (desired_shape_space_dimensionality + 2) columns:
%   if shape_index > (desired_shape_space_dimensionality + 2)
%     % continue
%     distance = nan;
%     % success = false;
%     success = true;
%     return
%   end

  % Check if work has been done
  imind2 = shape_index;
  imind1 = shape_index2;
  

  save_filename = [save_filename_prefix, filesep, num2str(imind2, '%d'), num2str(imind1, '_%d')]; 
  
  if useCurrentResults
    can_start = false;
  else
    [can_start, final_name, final_exists, tempname] = chunk_start(save_filename);
  end
  
  if ~can_start
%     stack = dbstack();
    % warning('Skipping "%s" at line %d in file %s.', save_filename, stack(1).line, stack(1).file)
    % fprintf('compute_distance_matrix: Skipping "%s" at line %d in file %s.\n', save_filename, stack(1).line, stack(1).file)
    if final_exists
      success = true;
      r = [];
      load([save_filename '.mat'], 'r')
      % distances(shape_index, shape_index2) = r.total_distance;
      distance = r.total_distance;
      return;
    else
      success = false;
      % warning('  Being computed elsewhere.')
      % fprintf('compute_distance_matrix: Being computed elsewhere.\n')
      % distances(shape_index, shape_index2) = nan;
      distance = NaN;
      return;
    end
  else
      
    fail = false;

    try
        [source] = image_function(shape_index); 
        [target] = image_function(shape_index2); 
    catch
        fail = true;
    end
      
    if fail | isempty(source) | isempty(target)
        disp(['Could not retreive images ' num2str(shape_index) ' and/or ' num2str(shape_index2)])
        distance = NaN;
        success = false;
    else

        fprintf('compute_distance_matrix: Computing %s\n', save_filename)
        % Get image


        % Perform work
        tic
        registration_options.convergence_registration_error = ...
            convergence_error_function(source, target);
        registration_options.single_sided = false;
        registration_options.just_compute_distance = true;
        % registration_options.verbose = 0;
        % registration_options
        
        r = Greedy3D_lambda_pre_compressed(...
          source, target, 1, registration_options)

        % Save work
        time_elapsed = toc;
        
        save([save_filename '.mat'], 'r', 'time_elapsed')

        distance = r.total_distance;
        success = true;

    end
    
    if ~useCurrentResults
        chunk_finish(tempname); 
    end
  end
end

function [ind1, ind2, isdone] = defaultCompletion(distances_sparse_file, dist_nans)
    if ~isempty(dist_nans)
        inds = sub2ind(size(distances_sparse_file), dist_nans(:,1), dist_nans(:,2));
        distances_sparse_file(inds) = 1;
        
    end

    inds = find(triu(~distances_sparse_file));
    
    if isempty(inds)
        isdone = true;
        ind1 = [];
        ind2 = [];
        return
    end
    
    ind = inds(1);
    isdone = false;
    [ind1, ind2] = ind2sub(size(distances_sparse_file), ind);
end

function [distances_incomplete, distances_sampled] = addCalculatedDistance(ind1, ind2, distance, distances_sparse_file)
    canstart = false;

    while ~canstart
        disp('Waiting for distance file lock')
        [canstart, ~, ~, temp_name] = chunk_start(distances_sparse_file, '');
        pause(1);
    end
    
    load(distances_sparse_file)
    
    distances_incomplete(ind1, ind2) = distance;
%     distances_incomplete(ind2, ind1) = distance;
    
    distances_sampled(ind1, ind2) = 1;
%     distances_sampled(ind2, ind1) = 1;
    
    disp('Writing updated distance file');
    save(distances_sparse_file, 'distances_incomplete', 'distances_sampled')
    
    disp('Releasing distance file')
    chunk_finish(temp_name);
    
end 

function [distances_incomplete, distances_sampled] = loadCalculatedDistances(number_shapes, distances_dir, distances_sparse_file)
    %if there is not a distances file and there is a lock
        %wait for it to show up (because someone else is assembling it)
    %end
    
    %if there is a distances file
        %load it
    %elseif there is a not a distances file
        %lock it
        %assemble it
    %end
        

    isunlocked = false;
    file_exists = exist(distances_sparse_file, 'file');
    temp_flag = false;

    while ~isunlocked && ~file_exists
        [isunlocked, ~, file_exists, temp_name] = chunk_start(distances_sparse_file, '');
        temp_flag = true;
        pause(1);
    end
    
    
    if file_exists
        disp('Loading existing distances file')
        
        load(distances_sparse_file)
    else
        disp('Constructing distance file from current results')
        
        distances_incomplete = sparse(number_shapes, number_shapes);
    
        distances_sampled = sparse(number_shapes, number_shapes);
        distances_sampled(logical(eye(size(distances_incomplete)))) = 1;
    
          
        files = dir([distances_dir filesep '*.mat']);
        files = {files.name};
        tokens = regexp(files, '[^0-9]', 'split');
        tokens = vertcat(tokens{:});

        if ~isempty(tokens)
            
            xy = str2double(tokens(:,1:2));
            x = xy(:,1);
            y = xy(:,2);


            for i = 1:size(xy(:,1))
                save_filename = [distances_dir filesep num2str(x(i)) '_' num2str(y(i)) '.mat'];

                load(save_filename, 'r')

                distances_incomplete(x(i), y(i)) = r.total_distance;
                distances_sampled(x(i), y(i)) = 1;

            end
        end
        save(distances_sparse_file, 'distances_incomplete', 'distances_sampled')
        
    end

    if temp_flag
        chunk_finish(temp_name);
    end
   
end
